xquery version "3.1";

module namespace import="http://ahikar.sub.uni-goettingen.de/ns/import";

import module namespace commons="http://ahikar.sub.uni-goettingen.de/ns/commons" at "commons.xqm";
import module namespace connect="https://sade.textgrid.de/ns/connect" at "/db/apps/textgrid-connect/modules/connect.xqm";
import module namespace rest="http://exquery.org/ns/restxq";

declare namespace output="http://www.w3.org/2010/xslt-xquery-serialization";

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/import-data-all")
    %rest:query-param("token", "{$token}")
    %output:method("json")
function import:main($token) {
    if( $token ne environment-variable("APP_DEPLOY_TOKEN" )) then
        error(QName("error://1", "deploy"), "Deploy token incorrect.")
    else
        let $log := local:log("STARTING IMPORT ALL RESOURCES")
        let $maps := 
            array {
                map {
                    "uri": "textgrid:3r132",
                    "recursive": "false",
                    "cleanup": "true", (: inital cleanup :)
                    "timeout": ()},
                map {
                    "uri": "textgrid:3r84g",
                    "recursive": "true",
                    "cleanup": "false",
                    "timeout": "30"},
                map {
                    "uri": "textgrid:3r84h",
                    "recursive": "true",
                    "cleanup": "false",
                    "timeout": "30"},
                map {
                    "uri": "textgrid:3r9ps",
                    "recursive": "true",
                    "cleanup": "false",
                    "timeout": "30"}
            }
    for $map in $maps?*
    return (
        local:log("IMPORTING " || $map("uri")), 
        let $url :=
            "http://existdb:8080/exist/restxq/import-data/" 
            || $map("uri")
            || "?token="
            || $token
            || "&amp;recursive="
            || $map("recursive")
            || "&amp;cleanup="
            || $map("cleanup")


        let $request := 
            element hc:request {
                attribute method {"GET"},
                attribute href {$url},
                if($map("timeout") ne "") then attribute timeout {$map("timeout")} else ()
            }
        return
            try {
                hc:send-request($request)
            } catch * {
                "started import with $request: " || serialize($request)
            }
            
)};

(:~
 : Remove all data present. Mainly used to cleanup sample data.
 : To not interfere with the collection trigger we are going to remove all
 : resources and not the collections themself.
 :)
declare function local:cleanup() {
    for $collection in ($commons:data, $commons:agg, $commons:meta, $commons:tile, $commons:json, $commons:html, $commons:tmp)
    where xmldb:collection-available($collection)
    let $resources := xmldb:get-child-resources($collection)
    for $resource in $resources
    return
        xmldb:remove($collection, $resource)
};

declare function local:publish($uri) {
    let $sid := commons:get-textgrid-session-id()
    let $user := "admin"
    let $password := environment-variable("EXIST_ADMIN_PW")
    return
        connect:publish($uri, $sid, $user, $password, false())
};

declare function local:log($message as xs:string) {
    util:log-system-out( util:system-time() || " ::: " || $message )
};

declare
    %rest:GET
    %rest:HEAD
    %rest:path("/import-data/{$uri}")
    %rest:query-param("token", "{$token}")
    %rest:query-param("cleanup", "{$cleanup}")
    %rest:query-param("recursive", "{$recursive}")
    %output:method("json")
function import:publish-uri($uri, $token, $cleanup, $recursive, $timeout) {
    if( $token ne environment-variable("APP_DEPLOY_TOKEN" )) then
        error(QName("error://1", "deploy"), "Deploy token incorrect.")
    else
        let $cleanup := if($cleanup eq "true") then local:cleanup() else ()
        let $sid := commons:get-textgrid-session-id()
        let $user := "admin"
        let $password := environment-variable("EXIST_ADMIN_PW")
        let $r := $recursive => xs:boolean()
        let $strictValidation := false()
        return
            connect:publish($uri, $sid, $user, $password, $strictValidation, $r)
};